@@ -10,7 +10,7 @@ mydata <- Spellman[,1:10]
 
 res <- mine(Spellman,master=1,n.cores=1,alpha=myalpha,C=myc)
 
-res <- mine(mydata, n.cores=1, var.thr=0.0)
+res <- mine(mydata, n.cores=1, var.thr=0.0, use="pair")
 
 ## aval.cores <- detectCores()
 ## if (aval.cores > 1){
